require 'spec_helper'
describe DidwwUps::CreditCard do
  before do

    @path = DidwwUps::CreditCard.site.to_s + DidwwUps::CreditCard.prefix.to_s
    @credit_card = DidwwUps::CreditCard.new({
                                                token_ref: 'LQ8IQ4Av6hM3tgX_WLJp9w'
                                            }, true)
  end

  context "Has properties" do
    before do
      @properties = {
          bin: "510510",
          bank: "BANK OF HAWAII",
          country: "UNITED STATES",
          card_brand: "MASTERCARD",
          card_type: "CREDIT",
          bank_phone: "1-888-643-3888 OR 1-888-643-9888",
          customer_ip: "195.138.65.189",
          fraud_level: nil}

      properties_uri_template = Addressable::Template.new @path + 'credit_cards/{id}/properties'


      stub_request(:get, properties_uri_template).to_return(body: @properties.to_json)

    end

    it "should respond to #properties and return credit card's properities" do
      expect(@credit_card.properties).to eq(@properties.stringify_keys)
    end

  end

  context "Has billing address" do
    before do
      @billing_address = {
          bill_first_name: "FirstName",
          bill_last_name: "SecondName",
          city: "Odessa",
          company: "",
          country: "UA",
          created_at: "2014-03-27T12:29:43Z",
          email: "olga@gmail.com",
          fax_number: "",
          first_address_line: "Address line1",
          id: 1070,
          phone_number: "3806600000000",
          second_address_line: "Address line2",
          state: nil,
          updated_at: "2014-03-27T12:29:43Z",
          zip: "65007",
          avs_result: nil
      }

      address_uri_template = Addressable::Template.new @path + 'credit_cards/{id}/billing_address'
      stub_request(:get, address_uri_template).to_return(body: @billing_address.to_json)
    end
    it "should respond to #billign_address and return credit card's billing address" do
      expect(@credit_card.billing_address).to eq(@billing_address.stringify_keys)
    end
  end

  context "Has modules" do

    before do
      @modules = ['Stripe', 'Braintree']

      modules_uri_template = Addressable::Template.new @path + 'credit_cards/{id}/modules'
      stub_request(:get, modules_uri_template).to_return(body: @modules.to_json)
    end

    it "should respond to #modules and return credit card's modules" do
      expect(@credit_card.modules).to eq(@modules)
    end
  end


  context "Read Collection" do
    before do
      @collection = [
          {token_ref: 'LQ8IQ4Av6hM3tgX_WLJp9w', credit_card_number: '555555******4444', credit_card_type: 'MASTERCARD', credit_card_expired: '2014-05', store_id: 1, customer_id: 1},
          {token_ref: 'LQ8IQ4Av6hM3tgX_WLJp9E', credit_card_number: '401288******1881', credit_card_type: 'VISA', credit_card_expired: '2015-09', store_id: 1, customer_id: 2}
      ]
      @collection_headers = {'X-Total-Count' => '10', 'X-Offset-Value' => '0', 'X-Limit-Value' => '2'}
      stub_request(:get, @path + 'credit_cards?page=1&per_page=2').to_return(body: @collection.to_json, headers: @collection_headers)
    end

    it "should be success" do
      credit_cards = DidwwUps::CreditCard.all(params: {page: 1, per_page: 2})
      expect(credit_cards.count).to eq(2)
      expect(credit_cards.total_count).to eq(@collection_headers['X-Total-Count'].to_i)
      expect(credit_cards.offset_value).to eq(@collection_headers['X-Offset-Value'].to_i)
      expect(credit_cards.limit_value).to eq(@collection_headers['X-Limit-Value'].to_i)

    end
  end

end