require 'spec_helper'
describe DidwwUps::Payment do
  before do
    @path = DidwwUps::Payment.site.to_s + DidwwUps::Payment.prefix.to_s
  end
  context "Read Collection" do
    before do


      @collection = [
          {id: 1967, token_ref: "DwyRv3iGrrrGvNtNJGHPUA", cc_type: "MASTERCARD", cc_number: "540400******0001", success: true, error: nil, amount: "20.5", order_id: "8", store_id: 6, module: "Stripe", operator: "Admin", description: "Payment for order # 8", customer_id: "143620"},
          {id: 1968, token_ref: "DwyRv3iGrrrGvNtNJGHPUA", cc_type: "MASTERCARD", cc_number: "540400******0001", success: true, error: nil, amount: "20.5", order_id: "9", store_id: 6, module: "Stripe", operator: "Admin", description: "Payment for order # 9", customer_id: "143620"}
      ]
      @collection_headers = {'X-Total-Count' => '10', 'X-Offset-Value' => '0', 'X-Limit-Value' => '2'}
      stub_request(:get, @path + 'payments?page=1&per_page=2').to_return(body: @collection.to_json, headers: @collection_headers)
    end

    it "should be success" do
      payments = DidwwUps::Payment.all(params: {page: 1, per_page: 2})
      expect(payments.count).to eq(2)
      expect(payments.total_count).to eq(@collection_headers['X-Total-Count'].to_i)
      expect(payments.offset_value).to eq(@collection_headers['X-Offset-Value'].to_i)
      expect(payments.limit_value).to eq(@collection_headers['X-Limit-Value'].to_i)

    end
  end

  context "Create Payment" do

    before do

      @credit_card = DidwwUps::CreditCard.new({
                                                  token_ref: 'LQ8IQ4Av6hM3tgX_WLJp9w'
                                              }, true)
      stub_request(:post, @path + 'payments').to_return(body: response.to_json, status: status)

      @payment = DidwwUps::Payment.create({token_ref: "LQ8IQ4Av6hM3tgX_WLJp9w", order_id: 7, amount: 20.5, operator: 'Admin', description: 'Payment for order # 7'})

    end

    let(:status) { 200 }


    context "Transaction failed" do
      let(:response) {
        {
            id: 1967,
            token_ref: "LQ8IQ4Av6hM3tgX_WLJp9w",
            cc_type: "MASTERCARD",
            cc_number: "540400******0001",
            success: false,
            error: "Declined",
            amount: 20.5,
            order_id: 7,
            description: "Payment for order # 7",
            operator: "Admin",
            type: "Stripe"
        }
      }

      it "should be created" do
        expect(@payment).to be_persisted

      end
      it "should be valid" do
        expect(@payment).to be_valid
      end
      it "should be failed" do
        expect(@payment).not_to be_success
      end
    end

    context "Invalid request" do
      let(:response) {
        {errors: {base: ["Order was already processed"]}}
      }
      let(:status) { 422 }
      it "should not be created" do
        expect(@payment).not_to be_persisted

      end
      it "should not be valid" do
        expect(@payment).not_to be_valid
      end

      it "should not be success" do
        expect(@payment).not_to be_success
      end


    end

    context "Transaction succeed" do

      let(:response) {
        {
            id: 1968,
            token_ref: "LQ8IQ4Av6hM3tgX_WLJp9w",
            cc_type: "MASTERCARD",
            cc_number: "540400******0001",
            success: true,
            error: nil,
            amount: 20.5,
            order_id: 7,
            description: "Payment for order # 7",
            operator: "Admin",
            type: "Stripe"}
      }


      it "should be created" do
        expect(@payment).to be_persisted

      end
      it "should be valid" do
        expect(@payment).to be_valid
      end
      it "should be success" do
        expect(@payment).to be_success
      end
    end


  end


end