require 'spec_helper'
describe DidwwUps::Refund do
  before do
    @path = DidwwUps::Refund.site.to_s + DidwwUps::Refund.prefix
  end
  context "Create Refund" do
    before do

      stub_request(:post, @path + 'refunds').to_return(body: response.to_json, status: status)

      @refund = DidwwUps::Refund.create(order_id: "8", amount: 12.00, operator: "Bob", description: "Stolen Card")

    end

    let(:status) { 200 }


    context "Transaction failed" do
      let(:response) {
        {
            id: 466,
            success: false,
            error: "Credit Card is expired",
            amount: 12.0,
            order_id: 8,
            description: "Stolen Card",
            operator: "Bob"
        }
      }

      it "should be created" do
        expect(@refund).to be_persisted

      end
      it "should be valid" do
        expect(@refund).to be_valid
      end
      it "should be failed" do
        expect(@refund).not_to be_success
      end
    end

    context "Invalid request" do
      let(:response) {
        {errors: {amount: ["must be less or equal to 10.0"]}}
      }
      let(:status) { 422 }
      it "should not be created" do
        expect(@refund).not_to be_persisted

      end
      it "should not be valid" do
        expect(@refund).not_to be_valid
      end

      it "should not be success" do
        expect(@refund).not_to be_success
      end


    end

    context "Transaction succeed" do

      let(:response) {
        {
            id: 466,
            success: true,
            error: nil,
            amount: 12.0,
            order_id: 8,
            description: "Stolen Card",
            operator: "Bob"
        }
      }


      it "should be created" do
        expect(@refund).to be_persisted

      end
      it "should be valid" do
        expect(@refund).to be_valid
      end
      it "should be success" do
        expect(@refund).to be_success
      end
    end
  end
  context "Read Collection" do
    before do
      @collection = [
          {id: 2, success: true, error: nil, amount: 10.01, order_id: 6, store_id: 1, description: 'Refund for order # 34', operator: 'Steve'},
          {id: 2, success: true, error: nil, amount: 10.02, order_id: 7, store_id: 1, description: 'Refund for order # 35', operator: 'Steve'}

      ]
      @collection_headers = {'X-Total-Count' => '10', 'X-Offset-Value' => '0', 'X-Limit-Value' => '2'}
      stub_request(:get, @path + 'refunds?page=1&per_page=2').to_return(body: @collection.to_json, headers: @collection_headers)
    end

    it "should be success" do
      credit_cards = DidwwUps::Refund.all(params: {page: 1, per_page: 2})
      expect(credit_cards.count).to eq(2)
      expect(credit_cards.total_count).to eq(@collection_headers['X-Total-Count'].to_i)
      expect(credit_cards.offset_value).to eq(@collection_headers['X-Offset-Value'].to_i)
      expect(credit_cards.limit_value).to eq(@collection_headers['X-Limit-Value'].to_i)

    end
  end


end