require 'spec_helper'
describe DidwwUps::Store do
  before do

    @store = DidwwUps.current_store

    @valid_params = callback_params.merge('secure_hash' => DidwwUps::CreditCard.new(callback_params).send(:valid_callback_hash))
  end
  #
  # ==== params
  # * +success+ - 0/1 (1 - YES)
  # * +credit_card_expired+ - (credit card expiration YEAR-month)
  # * +credit_card_number+ - (masked card number)
  # * +credit_card_type+ - (credit card type upcased)
  # * +customer_id+ - (customer id, that opened  the iframe  -  check if  currently logged in )
  # * +date_time+ - UTC datetime ()
  # * +result+ - =error string if success equal to 0 , credit card ups token other-way.
  # * +secure_hash+ - SHA1 of result + success + customer_id + date_time + api_secret_key
  let(:callback_params) {
    {
        'success' => '1',
        'result' => '4UHSmRkBMhWtHloKQfkjVQ',
        'customer_id' => 'john@doe.com',
        'credit_card_expired' => '2020-12',
        'credit_card_type' => 'MASTERCARD',
        'credit_card_number' => '540400******0001',
        'date_time' => '2014-06-06:09:19:16:000',

    }
  }


  context "Has no token" do

    let(:callback_params) {
      {
          'success' => '0',
          'result' => 'Some error',
          'customer_id' => 'john@doe.com',
          'credit_card_expired' => '',
          'credit_card_type' => '',
          'credit_card_number' => '',
          'date_time' => '2014-06-06:09:19:16:000',

      }
    }
    it "should retrieve invalid credit card" do

      credit_card = @store.retrieve_credit_card(@valid_params)
      expect(credit_card).not_to be_valid
      expect(credit_card.errors.full_messages).to eq([callback_params['result']])
    end

    context "Calling #retrieve_credit_card" do
      it "should raise InvalidRequest error" do
        expect { @store.retrieve_credit_card!(@valid_params) }.to raise_error(DidwwUps::Store::InvalidRequest, callback_params['result'])
      end

    end

  end


  context "Has success token" do


    context "Secure hash is valid" do
      it "should retrieve  credit card" do
        credit_card = @store.retrieve_credit_card(@valid_params)
        expect(credit_card).to be_valid
      end

    end

    context "Secure hash is invalid" do
      it "should retrieve invalid credit card" do
        params = @valid_params.merge('secure_hash' => "invalid")
        credit_card = @store.retrieve_credit_card(params)
        expect(credit_card).not_to be_valid
      end
    end


  end

  context "Build new credit card request" do
    before do
      @date_time = '2014-06-12 01:12:55'
      allow(@store).to receive(:formatted_date_time) { @date_time }


    end
    it "should build valid uri" do
      customer_id = 'some-unique_customer'
      card_id = "12"
      uri = @store.new_credit_card_uri(customer_id, {card_id: card_id})
      query = Rack::Utils.parse_nested_query(uri.query)
      expect(uri.host).to eq(DidwwUps::SANDBOX_HOST)
      expect(query['customer_id']).to eq(customer_id)
      expect(query['public_key']).to eq(@store['api_public_key'])
      expect(query['date_time']).to eq(@date_time)
      expect(query['hash']).to eq(@store.new_credit_card_uri_hash(customer_id, card_id, @date_time))
      expect(uri.path).to eq(DidwwUps::Store::NEW_CREDIT_CARD_PATH)

    end

  end

end
